@@ -7,6 +7,7 @@ from __future__ import division, print_function
 import os.path
 import collections
 import yaml
+from pymatgen.io.abinitio import myaml
 
 from pymatgen.util.string_utils import WildCard
 from pymatgen.io.abinitio.abiinspect import YamlTokenizer, YamlDoc
@@ -326,15 +327,16 @@ class EventsParser(object):
                 if w.match(doc.tag):
                     #print("got doc.tag", doc.tag,"--")
                     try:
-                        event = yaml.load(doc.text)
+                        event = myaml.load(doc.text)
                     except:
                         # Wrong YAML doc. Check tha doc tag and instantiate the proper event.
                         message = "Malformatted YAML document at line: %d\n" % doc.lineno
                         message += doc.text
-                        message += "Traceback:\n %s" % straceback()
+                        # This call is very expensive when we have many exceptions due to malformatted YAML docs. 
+                        #message += "Traceback:\n %s" % straceback()
 
                         if "error" in doc.tag.lower():
-                            print("seems an error",doc.tag)
+                            print("It seems an error",doc.tag)
                             event = AbinitYamlError(message=message, src_file=__file__, src_line=0)
                         else:
                             event = AbinitYamlWarning(message=message, src_file=__file__, src_line=0)
